import logging
import sys
from pathlib import Path
from time import sleep

from tqdm import tqdm

from flach.utils import build_name, fetch_files, project, write_projection, parse_args

logging.basicConfig(format='%(levelname)s: %(message)s')

logger_cli = logging.getLogger(__name__)
logger_cli.setLevel(logging.INFO)

logger_tifffile = logging.getLogger("tifffile")
logger_tifffile.setLevel(logging.ERROR)


def cli():
    args = parse_args()
    if args.mock:
        logger_cli.warning('Mock mode: Files are not processed.')
    path_source = Path(args.source)
    files = fetch_files(path_source, recursive=args.recursive)

    if args.top:
        projections_path = path_source / 'projections'
        projections_path.mkdir(exist_ok=True)
    else:
        projections_path = None
    logger_cli.info('Projections will be saved in %s', projections_path)

    if files:
        logger_cli.info('Found %s files', len(files))
    else:
        logger_cli.warning('No file found...')
        logger_cli.info('The recursive flag is currently %s', args.recursive)

    pbar = tqdm(files)

    for path in pbar:
        logging.info('Processing: %s', path.name)
        dst_name = build_name(path)

        if not Path(path_source / 'raw').exists():
            sys.exit('Please move the raw ome.tif files or tree thereof to `raw/`')

        if not args.top:
            projections_path = path

        path_dst = projections_path / dst_name

        if not args.force and path_dst.exists():
            logging.warning('Projection already exists (%s)', path_dst)
            should_overwrite = input('Do you want to overwrite the existing projection? (y/n): ')
            if should_overwrite in ('n', 'no'):
                logging.warning('Skipping %s', path)
                continue

        if args.mock:
            sleep(.1)
            continue

        pixel_size_cm, projection = project(path)
        write_projection(path_dst, projection, pixel_size_cm)


if __name__ == '__main__':
    cli()
