import argparse
import logging
import pathlib

import tifffile as tf


def setup_projection_folder():
    pass


def parse_args():
    parser = argparse.ArgumentParser(allow_abbrev=True,
                                     description='Project OME.tiff files',
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('source',
                        type=pathlib.Path,
                        help='Path to the dataset folder; the parent of `raw`.',
                        )
    parser.add_argument('--top', '-t',
                        action='store_true',
                        help='Write the projections in one single folder `projections` immediately below the source.',
                        )
    parser.add_argument('--recursive', '-r',
                        action='store_true',
                        default=False,
                        help='Process all ome.tif files under the directory.',
                        )
    parser.add_argument('--force', '-f',
                        action='store_true',
                        )
    parser.add_argument('--mock', '-m',
                        action='store_true',
                        help='Emulate the iteration with no heavy I/O nor max projection.'
                        )

    return parser.parse_args()


def fetch_files(path_source, file_type='.ome.tif', recursive=False):
    """
    Collect all ome.tif files in a list.
    :param file_type:
    :param path_source:
    :param recursive:
    :return: A list of Path to ome.tif files
    """
    pattern = f'*{file_type}'

    if recursive:
        files_generator = path_source.rglob(pattern)
    else:
        files_generator = path_source.glob(pattern)

    return [file for file in files_generator if not file.name.startswith('.')]


def build_name(path):
    """
    Remove the suffixes and append `_max`.
    :param path:
    :return:
    """
    file_name = path.name
    suffixes = ''.join(path.suffixes)
    file_name_no_suffix = file_name.rstrip(suffixes)
    return file_name_no_suffix + '_max' + '.tif'


def project(path):
    """
    Reads an OME.tiff and applies a max projection for the z-axis.
    :returns the pixel size in centimeters and the CYX-array.
    """
    with tf.TiffFile(path) as file:
        shape = file.series[0].shape
        order = file.series[0].axes
        dimensions_found = set(order.lower())
        dimensions_expected = set('czyx')

        if dimensions_found != dimensions_expected:
            raise ValueError(f"Dimension mismatch: found: {dimensions_found} vs expected: {dimensions_expected}")
        if order == 'ZCYX':
            z, c, y, x = shape
        elif order == 'CZYX':
            c, z, y, x = shape
        else:
            raise ValueError(f'Order is not understood {order}')

        try:
            micromanager_metadata = file.micromanager_metadata
            pixel_size_um = micromanager_metadata['Summary']['PixelSize_um']
        except AttributeError:
            pixel_size_um = None
            logging.warning('No pixel size could be found')

        if pixel_size_um:
            pixel_size_cm = pixel_size_um / 1e4
        else:
            pixel_size_cm = None

        data = file.asarray()

        logging.info('Order: %s Shape: %s', order, shape)
        _data = data.reshape((c, z, y, x))

    projection = _data.max(axis=1)

    return pixel_size_cm, projection


def write_projection(dst, data, pixel_size=None):
    """
    Writes the projection to the disk.
    """
    if pixel_size:
        res = (1 / pixel_size, 1 / pixel_size, 'CENTIMETER')
    else:
        res = None
    tf.imwrite(dst, data, photometric='minisblack', resolution=res)
